#ifndef _PQ_H_
#define _PQ_H_

#pragma ident "@(#)pq.h	1.1 12/12/96 Voronoi"
// Dec 1996
// Copyright 1996 SFBellenot All rights reserved

#include "halfedge.h"
#include <string.h>

//template on HalfEdge?

struct PQnode
{
	HalfEdge* he;
	PQnode* next;
};

class PQ
{
public:
	PQ ( int SqrtNumberOfSites, Real YMin, Real YMax )
	{
		hashSize = 4 * SqrtNumberOfSites;
		count = 0;
		min = 0;
		yMin = YMin;
		deltaY = YMax - YMin;

		hash = new PQnode* [hashSize];
		memset ( hash, 0, hashSize*sizeof(PQnode*) );
	}
	~PQ()
	{
		int i;

		for ( i = 0; i < hashSize; i++ )
		{
			PQnode* last = hash[i];
			PQnode* next;

			while ( last )
			{
				if ( last->he )
					last->he->Deref();
				next = last->next;
				delete last;
				last = next;
			}
		}
		delete[] hash;
	}
	void Insert ( HalfEdge* he, Vertex* v, Real offset );
	void Delete ( HalfEdge* he );
	int Empty() { return (count == 0); }
	Point MinPoint();
	HalfEdge* ExtractMin ();
	void Print();
	
private:
	int Bucket ( HalfEdge* he )
	{
		int bucket = (he->YStar() - yMin)/ deltaY * hashSize;
		if ( bucket < 0 )
			bucket = 0;
		if ( bucket >= hashSize )
			bucket = hashSize - 1;
		if ( bucket < min )
			min = bucket;
		return bucket;
	}
	int count;
	int min;
	int hashSize;
	Real yMin;
	Real deltaY;
	PQnode** hash;
};

#endif
